/** Source code example for "A Practical Introduction to Data
    Structures and Algorithm Analysis, 3rd Edition (Java)" 
    by Clifford A. Shaffer, Prentice Hall, 2009.
    Copyright 2008 by Clifford A. Shaffer
*/

/** Dictionary implemented using a B+ tree. */
class BPTree<K extends Comparable<? super K>, E>
                    implements Dictionary<K, E> {
  private BPNode<K,E> root;     // Root of B+ tree
  private int nodecount;        // # of records now in 2-3 tree
  private static final int NUMRECS = 10;

  /** Stub for a binary search on the key array */
  private int binaryle(K keys[], int cnt, K k) { return 0; }

  BPTree() { 
    nodecount = 0;
    root = new BPLeaf<K,E>(NUMRECS);
  }

  public void clear() {   /** Reinitialize */
    nodecount = 0;
    root = null;
  }

  public void insert(K k, E e) {  /** Insert an element */
    root = inserthelp(root, k, e);
    nodecount++;
  }

  /** Remove an element */
  public E remove(K k) {
    E temp = findhelp(root, k);
    if (temp != null) {
      if (removehelp(root, k) && (root.numrecs() == 1)) // Collapse root
        if (!root.isLeaf()) root = ((BPInternal<K,E>)root).pointers(0);
      nodecount--;
    }
    return temp;
  }

  /** Remove some element. */
  public E removeAny() {
    System.out.println("removeany not implemented");
    return null;
  }

  /** Find a record with key value "k" */
  public E find(K k) {
    return findhelp(root, k);
  }

  /** Return number of values in the tree */
  public int size() { return nodecount; }

private BPNode<K,E> inserthelp(BPNode<K,E> rt, K k, E e) {
  BPNode<K,E> retval;
  if (rt.isLeaf()) // At leaf node: insert here
    return ((BPLeaf<K,E>)rt).add(k, e);
  // Add to internal node
  int currec = binaryle(rt.keys(), rt.numrecs(), k);
  BPNode<K,E> temp = inserthelp(
            ((BPInternal<K,E>)root).pointers(currec), k, e);
  if (temp != ((BPInternal<K,E>)rt).pointers(currec))
    return ((BPInternal<K,E>)rt).add((BPInternal<K,E>)temp);
  else
    return rt;
}
private E findhelp(BPNode<K,E> rt, K k) {
  int currec = binaryle(rt.keys(), rt.numrecs(), k);
  if (rt.isLeaf())
    if ((((BPLeaf<K,E>)rt).keys())[currec].compareTo(k)
        == 0)
      return ((BPLeaf<K,E>)rt).recs(currec);
    else return null;
  else
    return findhelp(((BPInternal<K,E>)rt).pointers(currec),
                    k);
}
/** Delete a record with the given key value, and 
    return true if the root underflows */
private boolean removehelp(BPNode<K,E> rt, K k) {
  int currec = binaryle(rt.keys(), rt.numrecs(), k);
  if (rt.isLeaf())
    if ((((BPLeaf<K,E>)rt).keys()[currec]).compareTo(k)
        == 0)
      return ((BPLeaf<K,E>)rt).delete(currec);
    else return false;
  else // Process internal node
    if (removehelp(((BPInternal<K,E>)rt).pointers(currec),
                   k))
      // Child will merge if necessary
      return ((BPInternal<K,E>)rt).underflow(currec);
    else return false;
}
}
