/** Source code example for "A Practical Introduction to Data
    Structures and Algorithm Analysis, 3rd Edition (Java)" 
    by Clifford A. Shaffer, Prentice Hall, 2009.
    Copyright 2008 by Clifford A. Shaffer
*/

// Doubly linked list implementation
class LList<E> implements List<E> {
private DLink<E> head;        // Pointer to list header
private DLink<E> tail;        // Pointer to last element in list 
protected DLink<E> curr;      // Pointer ahead of current element
int cnt;		      // Size of list

//Constructors
LList(int size) { this(); }  // Ignore size
LList() {
  curr = tail = head = new DLink<E>(null, null); // Create header node
  cnt = 0;
}

public void clear() {         // Remove all elements from list
  head.setNext(null);         // Drop access to rest of links
  curr = tail = head = new DLink<E>(null, null); // Create header node
  cnt = 0;
}

public void moveToStart()  // Set curr at list start
{ curr = head; }

public void moveToEnd()  // Set curr at list end
{ curr = tail; }
// Insert "it" at current position
public void insert(E it) {
  curr.setNext(new DLink<E>(it, curr.next(), curr));  
  if (curr.next().next() != null)
    curr.next().next().setPrev(curr.next());
  if (tail == curr) tail = curr.next();  // New tail
  cnt++;
}

public void append(E it) { // Append "it" to list
  tail.setNext(new DLink<E>(it, null, tail));
  tail = tail.next();
  cnt++;
}

// Remove and return first element in right partition
public E remove() {
  if (curr.next() == null) return null; // Nothing to remove
  E it = curr.next().element();      // Remember value
  if (curr.next().next() != null)
    curr.next().next().setPrev(curr);
  else tail = curr;         // Removed last Object: set tail
  curr.setNext(curr.next().next());  // Remove from list
  cnt--;			     // Decrement the count
  return it;                         // Return value removed
}

// Move curr one step left; no change if at front
public void prev() {
  if (curr != head) curr = curr.prev();
}
// Move curr one step right; no change if at end
public void next()
  { if (curr != tail) curr = curr.next(); }

public int length() { return cnt; }

// Return the position of the current element
public int currPos() {
  DLink<E> temp = head;
  int i;
  for (i=0; curr != temp; i++)
    temp = temp.next();
  return i;
}

// Move down list to "pos" position
public void moveToPos(int pos) {
  assert (pos>=0) && (pos<=cnt) : "Position out of range";
  curr = head;
  for(int i=0; i<pos; i++) curr = curr.next();
}

public E getValue() {   // Return current element
  assert curr.next() != null : "Nothing to get";
  return curr.next().element();
}
// Extra stuff not printed in the book.

  /**
   * Generate a human-readable representation of this list's contents
   * that looks something like this: < 1 2 3 | 4 5 6 >.  The vertical
   * bar represents the current location of the fence.  This method
   * uses toString() on the individual elements.
   * @return The string representation of this list
   */
  public String toString()
  {
    // Save the current position of the list
    int oldPos = currPos();
    int length = length();
    StringBuffer out = new StringBuffer((length() + 1) * 4);

    moveToStart();
    out.append("< ");
    for (int i = 0; i < oldPos; i++) {
      out.append(getValue());
      out.append(" ");
      next();
    }
    out.append("| ");
    for (int i = oldPos; i < length; i++) {
      out.append(getValue());
      out.append(" ");
      next();
    }
    out.append(">");
    moveToPos(oldPos); // Reset the fence to its original position
    return out.toString();
  }
}
